/*****************************************************************************
 *                                                                           *
 * File:                                                                     *
 *  toedev.h                                                                 *
 *                                                                           *
 * Description:                                                              *
 *  TOE device definitions.                                                  *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License, version 2, as       *
 * published by the Free Software Foundation.                                *
 *                                                                           *
 * You should have received a copy of the GNU General Public License along   *
 * with this program; if not, write to the Free Software Foundation, Inc.,   *
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                 *
 *                                                                           *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED    *
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF      *
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.                     *
 *                                                                           *
 * http://www.chelsio.com                                                    *
 *                                                                           *
 * Copyright (c) 2003 - 2005 Chelsio Communications, Inc.                    *
 * All rights reserved.                                                      *
 *                                                                           *
 * Maintainers: maintainers@chelsio.com                                      *
 *                                                                           *
 * Authors: Dimitrios Michailidis   <dm@chelsio.com>                         *
 *                                                                           *
 * History:                                                                  *
 *                                                                           *
 ****************************************************************************/
/* $Date: 2005/11/18 07:48:51 $ $RCSfile: toedev.h,v $ $Revision: 1.19 $ */

#ifndef _TOEDEV_H_
#define _TOEDEV_H_

#include <linux/list.h>
#include <asm/atomic.h>
#include <linux/netdevice.h>
#include <linux/proc_fs.h>
#include <linux/skbuff.h>
#include <net/neighbour.h>
#include <net/sock.h>

#define TOENAMSIZ 16

/* belongs in linux/netdevice.h */
#define NETIF_F_TCPIP_OFFLOAD (1 << 16)

/* Get the toedev associated with a net_device */
#define TOEDEV(netdev) (*(struct toedev **)&(netdev)->ec_ptr)

/* TOE type ids */
enum {
	TOE_ID_CHELSIO_T1  = 1,
	TOE_ID_CHELSIO_T1C,
	TOE_ID_CHELSIO_T3,
};

struct toe_id {
	unsigned int id;
	unsigned long data;
};

struct tom_info;

struct toedev {
	char name[TOENAMSIZ];               /* TOE device name */
	struct list_head toe_list;          /* for list linking */
	int toe_index;                      /* unique TOE device index */
	unsigned int ttid;                  /* TOE type id */
	unsigned long flags;                /* device flags */
	unsigned int mtu;                   /* max size of TX offloaded data */ 
	unsigned int nconn;                 /* max # of offloaded connections */
	struct net_device *lldev;   /* LL device associated with TOE messages */
	const struct tom_info *offload_mod; /* attached TCP offload module */
	struct proc_dir_entry *proc_dir;    /* root of proc dir for this TOE */
	int (*open)(struct toedev *dev);
	int (*close)(struct toedev *dev);
	int (*can_offload)(struct toedev *dev, struct sock *sk);
	int (*connect)(struct toedev *dev, struct sock *sk);
	int (*send)(struct toedev *dev, struct sk_buff *skb);
	int (*recv)(struct toedev *dev, struct sk_buff **skb, int n);
	int (*ctl)(struct toedev *dev, unsigned int req, void *data);
	void (*neigh_update)(struct toedev *dev, struct neighbour *neigh, 
			     int fl, struct net_device *lldev);
	void *priv;                         /* driver private data */
	void *l2opt;                        /* optional layer 2 data */
	void *l3opt;                        /* optional layer 3 data */
	void *l4opt;                        /* optional layer 4 data */
	void *ulp;				/* ulp stuff */
	atomic_t refcnt;                    /* reference count */
};

struct tom_info {
	int (*attach)(struct toedev *dev, const struct toe_id *entry);
	int (*detach)(struct toedev *dev);
	const char *name;
	struct toe_id *id_table;
	struct list_head list_node;
};

/* Flags for toe_neigh_update() */
enum {
	NEIGH_ADDR_CHANGED = 1
};

static inline void toedev_hold(struct toedev *dev)
{
	atomic_inc(&dev->refcnt);
}

static inline void toedev_put(struct toedev *dev)
{
	atomic_dec(&dev->refcnt);
}

int register_tom(struct tom_info *t);
int unregister_tom(struct tom_info *t);
int register_toedev(struct toedev *dev, const char *name);
int activate_toedev(struct toedev *dev);
struct toedev *alloc_toedev(void);
void toe_set_lldev(struct toedev *dev, struct net_device *lldev);
int toe_send(struct toedev *dev, struct sk_buff *skb);
int toe_receive_skb(struct toedev *dev, struct sk_buff **skb, int n);
void toe_neigh_update(struct neighbour *neigh, int flags);
#endif /* _TOEDEV_H_ */
